@@ -8,17 +8,43 @@
 use std::collections::BTreeSet;
 
 use serde::{Deserialize, Serialize};
-use threshold_crypto::PublicKeySet;
+use threshold_crypto::PublicKey;
 use tiny_keccak::{Hasher, Sha3};
 
-use crate::DbcContentHash;
+use crate::{DbcContentHash, Error, Hash};
+
+#[derive(Debug, PartialEq, Eq, Hash, Clone, Serialize, Deserialize)]
+pub struct BlindedOwner(Hash);
+
+impl BlindedOwner {
+    pub fn new(
+        owner: &PublicKey,
+        parents: &BTreeSet<DbcContentHash>,
+        amount: u64,
+        output_number: u8,
+    ) -> Self {
+        let mut sha3 = Sha3::v256();
+
+        for parent in parents.iter() {
+            sha3.update(parent);
+        }
+
+        sha3.update(&amount.to_be_bytes());
+        sha3.update(&output_number.to_be_bytes());
+        sha3.update(&owner.to_bytes());
+
+        let mut hash = [0; 32];
+        sha3.finalize(&mut hash);
+        Self(Hash(hash))
+    }
+}
 
 #[derive(Debug, PartialEq, Eq, Hash, Clone, Serialize, Deserialize)]
 pub struct DbcContent {
     pub parents: BTreeSet<DbcContentHash>, // Parent DBC's, acts as a nonce
     pub amount: u64,
     pub output_number: u8,
-    pub owner: PublicKeySet, // TAI: should this be threshold_crypto::PublicKey? (PublicKeySet::public_key())
+    pub owner: BlindedOwner,
 }
 
 impl DbcContent {
@@ -27,12 +53,9 @@ impl DbcContent {
         parents: BTreeSet<DbcContentHash>,
         amount: u64,
         output_number: u8,
-        owner: PublicKeySet,
+        owner_key: PublicKey,
     ) -> Self {
-        // let mut owner = owner;
-        // for _ in 0..amount % 1000 {
-        //     owner = sha3_256(&owner); // owner not visible to mint, until out_dbc is minted.
-        // }
+        let owner = BlindedOwner::new(&owner_key, &parents, amount, output_number);
         DbcContent {
             parents,
             amount,
@@ -41,10 +64,16 @@ impl DbcContent {
         }
     }
 
-    pub fn hash(&self) -> DbcContentHash {
-        // let data = serde_json::to_string(&self)?; // use the sha3 256 of the json string repr for x platform use
-        // Ok(sha3_256(data.as_ref()))
+    pub fn validate_unblinding(&self, owner_key: &PublicKey) -> Result<(), Error> {
+        let blinded = BlindedOwner::new(owner_key, &self.parents, self.amount, self.output_number);
+        if blinded == self.owner {
+            Ok(())
+        } else {
+            Err(Error::FailedUnblinding)
+        }
+    }
 
+    pub fn hash(&self) -> DbcContentHash {
         let mut sha3 = Sha3::v256();
 
         for parent in self.parents.iter() {
@@ -53,9 +82,10 @@ impl DbcContent {
 
         sha3.update(&self.amount.to_be_bytes());
         sha3.update(&self.output_number.to_be_bytes());
+        sha3.update(&self.owner.0);
 
         let mut hash = [0; 32];
         sha3.finalize(&mut hash);
-        hash
+        Hash(hash)
     }
 }
